Package: index
Title: Test package for document/article indices
Version: 0.0.0.9000
Authors@R: 
    c(person(given = "Hadley",
             family = "Wickham",
             role = c("aut", "cre"),
             email = "hadley@rstudio.com"),
      person(given = "RStudio",
             role = "cph"))
Description: What the package does (one paragraph).
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.1.0
Suggests: 
    knitr,
    rmarkdown
VignetteBuilder: knitr
