# Generated by roxygen2: do not edit by hand

S3method(is_low_change,default)
S3method(replay_html,"NULL")
S3method(replay_html,character)
S3method(replay_html,error)
S3method(replay_html,html)
S3method(replay_html,htmlwidget)
S3method(replay_html,list)
S3method(replay_html,message)
S3method(replay_html,recordedplot)
S3method(replay_html,shiny.tag)
S3method(replay_html,shiny.tag.function)
S3method(replay_html,shiny.tag.list)
S3method(replay_html,source)
S3method(replay_html,warning)
export(autolink)
export(autolink_url)
export(classes_chroma)
export(classes_pandoc)
export(downlit_html_node)
export(downlit_html_path)
export(downlit_md_path)
export(downlit_md_string)
export(evaluate_and_highlight)
export(highlight)
export(href_article)
export(href_package)
export(href_topic)
export(is_low_change)
import(rlang)
